@@ -24,6 +24,7 @@ class MqttAsyncClient():
 
     # Errors from mqtt module - mirrored into this class
     MQTT_ERR_SUCCESS = mqtt.MQTT_ERR_SUCCESS
+    MQTT_ERR_NO_CONN = mqtt.MQTT_ERR_NO_CONN
 
     log = logging.getLogger('cloudio.mqttasyncclient')
 
@@ -125,10 +126,17 @@ class MqttAsyncClient():
 
             self._client.connect(self._host, port=port)
             self._client.loop_start()
+            time.sleep(1)  # Wait a bit for the callback onConnect to be called
         self._clientLock.release()
 
-    def disconnect(self):
+    def disconnect(self, force_client_disconnect=True):
         """Disconnects MQTT client
+
+        In case to let MQTT client die silently, call force_client_disconnect parameter with
+        'false' value. In this case no disconnect callback method is called.
+
+        ::param force_client_disconnect Set to true to call also MQTT clients disconnect method. Default: true
+        :type force_client_disconnect bool
         """
         self._isConnected = False
 
@@ -136,12 +144,13 @@ class MqttAsyncClient():
         # Stop MQTT client if still running
         if self._client:
             self._client.loop_stop()
-            self._client.disconnect()
+            if force_client_disconnect:
+                self._client.disconnect()
             self._client = None
         self._clientLock.release()
 
     def isConnected(self):
-        return self._isConnected
+        return self._client and self._isConnected
 
     def onConnect(self, client, userdata, flags, rc):
         if rc == 0:
@@ -201,7 +210,11 @@ class MqttAsyncClient():
         return message_info.rc == self.MQTT_ERR_SUCCESS
 
     def subscribe(self, topic, qos=0):
-        return self._client.subscribe(topic, qos)
+        if self._client:
+            return self._client.subscribe(topic, qos)
+        else:
+            return (self.MQTT_ERR_NO_CONN, None)
+
 
 class MqttReconnectClient(MqttAsyncClient):
     """Same as MqttAsyncClient, but adds reconnect feature.
@@ -300,7 +313,7 @@ class MqttReconnectClient(MqttAsyncClient):
                 traceback.print_exc()
                 self.log.warning(u'Error during broker connect!')
                 # Force disconnection of MQTT client
-                self.disconnect()
+                self.disconnect(force_client_disconnect=False)
                 # Do not exit here. Continue to try to connect
 
             # Check if thread should leave
